@@ -1,6 +1,6 @@
 /* pk-cmd.c - Poke commands.  */
 
-/* Copyright (C) 2019, 2020, 2021, 2022, 2023 Jose E. Marchesi */
+/* Copyright (C) 2019, 2020, 2021, 2022, 2023, 2024 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -48,6 +48,9 @@ extern const struct pk_cmd mem_cmd; /* pk-cmd-ios.c */
 #ifdef HAVE_LIBNBD
 extern const struct pk_cmd nbd_cmd; /* pk-cmd-ios.c */
 #endif
+#ifdef HAVE_MMAP
+extern const struct pk_cmd mmap_cmd; /* pk-cmd-mmap.c */
+#endif
 extern const struct pk_cmd close_cmd; /* pk-cmd-file.c */
 extern const struct pk_cmd load_cmd; /* pk-cmd-file.c */
 extern const struct pk_cmd source_cmd; /* pk-cmd-ios.c */
@@ -59,6 +62,7 @@ extern const struct pk_cmd doc_cmd; /* pk-cmd-misc.c */
 extern const struct pk_cmd jmd_cmd; /* pk-cmd-misc.c */
 extern const struct pk_cmd help_cmd; /* pk-cmd-help.c */
 extern const struct pk_cmd vm_cmd; /* pk-cmd-vm.c  */
+extern const struct pk_cmd compiler_cmd; /* pk-cmd-compiler.c */
 extern struct pk_cmd set_cmd; /* pk-cmd-set.c */
 extern const struct pk_cmd editor_cmd; /* pk-cmd-editor.c */
 extern const struct pk_cmd map_cmd; /* pk-cmd-map.c */
@@ -82,12 +86,16 @@ static const struct pk_cmd *dot_cmds[] =
     &source_cmd,
     &help_cmd,
     &vm_cmd,
+    &compiler_cmd,
     &set_cmd,
     &map_cmd,
     &editor_cmd,
     &mem_cmd,
 #ifdef HAVE_LIBNBD
     &nbd_cmd,
+#endif
+#ifdef HAVE_MMAP
+    &mmap_cmd,
 #endif
     &null_cmd
   };
@@ -401,22 +409,6 @@ pk_cmd_exec_1 (const char *str, struct pk_trie *cmds_trie, char *prefix)
                         }
                     }
 
-                  break;
-                case 't':
-                  /* Parse a #N tag.  */
-                  p = skip_blanks (p);
-                  if (*p == '#'
-                      && p++
-                      && pk_atoi (&p, &(argv[argc].val.tag))
-                      && argv[argc].val.tag >= 0)
-                    {
-                      if (*p == ',' || *p == '\0' || isblank (*p))
-                        {
-                          argv[argc].type = PK_CMD_ARG_TAG;
-                          match = 1;
-                        }
-                    }
-
                   break;
                 case 's':
                   {
@@ -580,6 +572,9 @@ extern struct pk_trie *info_trie; /* pk-cmd-info.c  */
 extern const struct pk_cmd *vm_cmds[]; /* pk-cmd-vm.c  */
 extern struct pk_trie *vm_trie;  /* pk-cmd-vm.c  */
 
+extern const struct pk_cmd *compiler_cmds[]; /* pk-cmd-compiler.c */
+extern struct pk_trie *compiler_trie; /* pk-cmd-compiler.c */
+
 extern const struct pk_cmd *vm_disas_cmds[];  /* pk-cmd-vm.c */
 extern struct pk_trie *vm_disas_trie; /* pk-cmd-vm.c */
 
@@ -763,9 +758,12 @@ pk_cmd_exec_script (const char *filename)
 void
 pk_cmd_init (void)
 {
+  pk_val exception;
+
   cmds_trie = pk_trie_from_cmds (dot_cmds);
   info_trie = pk_trie_from_cmds (info_cmds);
   vm_trie = pk_trie_from_cmds (vm_cmds);
+  compiler_trie = pk_trie_from_cmds (compiler_cmds);
   vm_disas_trie = pk_trie_from_cmds (vm_disas_cmds);
   vm_profile_trie = pk_trie_from_cmds (vm_profile_cmds);
   map_trie = pk_trie_from_cmds (map_cmds);
@@ -777,8 +775,13 @@ pk_cmd_init (void)
   set_cmd.subcommands = set_cmds;
 
   /* Compile commands written in Poke.  */
-  if (pk_load (poke_compiler, "pk-cmd") != PK_OK)
-    pk_fatal ("unable to load the pk-cmd module");
+  if (pk_load (poke_compiler, "pk-cmd", &exception) != PK_OK)
+    pk_fatal ("unable to load the pk-cmd module due to compile-time error");
+  else if (exception != PK_NULL)
+    {
+      poke_handle_exception (exception);
+      pk_fatal ("unable to load the pk-cmd module due to run-time exception");
+    }
 }
 
 void
@@ -787,6 +790,7 @@ pk_cmd_shutdown (void)
   pk_trie_free (cmds_trie);
   pk_trie_free (info_trie);
   pk_trie_free (vm_trie);
+  pk_trie_free (compiler_trie);
   pk_trie_free (vm_disas_trie);
   pk_trie_free (vm_profile_trie);
   pk_trie_free (set_trie);
