@@ -33,6 +33,7 @@
 #include <poll.h>
 #include <pthread.h>
 #include <sys/socket.h>
+#include <sys/stat.h>
 #include <sys/un.h>
 #include <unistd.h>
 
@@ -622,6 +623,7 @@ usock_new (const char *path)
   struct usock *u;
   struct sockaddr_un adr;
   int flags;
+  mode_t mask;
 
   u = calloc (1, sizeof (*u));
   if (u == NULL)
@@ -634,15 +636,10 @@ usock_new (const char *path)
 
   u->pipefd[0] = -1;
   u->pipefd[1] = -1;
-  if (pipe (u->pipefd) == -1)
+  if (pipe2 (u->pipefd, O_CLOEXEC) == -1)
     goto error;
-  if ((flags = fcntl (u->pipefd[0], F_GETFD)) == -1
-      || fcntl (u->pipefd[0], F_SETFD, flags | FD_CLOEXEC) == -1)
-    goto error;
-  if ((flags = fcntl (u->pipefd[1], F_GETFD)) == -1
-      || fcntl (u->pipefd[1], F_SETFD, flags | FD_CLOEXEC) == -1)
-    goto error;
-  if (fcntl (u->pipefd[0], F_SETFL, O_NONBLOCK) == -1)
+  if ((flags = fcntl (u->pipefd[0], F_GETFL)) == -1
+      || fcntl (u->pipefd[0], F_SETFL, flags | O_NONBLOCK) == -1)
     goto error;
 
   if ((u->fd = socket (AF_UNIX, SOCK_STREAM, 0)) == -1)
@@ -651,15 +648,18 @@ usock_new (const char *path)
   if ((flags = fcntl (u->fd, F_GETFD)) == -1
       || fcntl (u->fd, F_SETFD, flags | FD_CLOEXEC) == -1)
     goto error;
-  if (fcntl (u->fd, F_SETFL, O_NONBLOCK) == -1)
+  if ((flags = fcntl (u->fd, F_GETFL)) == -1
+      || fcntl (u->fd, F_SETFL, flags | O_NONBLOCK) == -1)
     goto error;
 
   memset (&adr, 0, sizeof (adr));
   adr.sun_family = AF_UNIX;
   strncpy (adr.sun_path, path, sizeof (adr.sun_path) - 1);
   unlink (adr.sun_path);
+  mask = umask (0077);
   if (bind (u->fd, (struct sockaddr *)&adr, sizeof (adr)) == -1)
     goto error;
+  umask (mask);
   if (listen (u->fd, 128) == -1)
     goto error;
 
